#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) since 2015 NTT DATA Corporation 
# Last update : 2015/09/15
#
from __future__ import unicode_literals

import argparse
import codecs
import docker
import logging.config
import os
import sys
import threading
import traceback
import re

SCRIPT_NAME = os.path.splitext(os.path.basename(sys.argv[0]))[0]
DOCKER_API_VERSION = "1.18"
DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = "2375"

# 標準入出力をUTF-8で行う
sys.stdin = codecs.getreader("utf-8")(sys.stdin)
sys.stdout = codecs.getwriter("utf-8")(sys.stdout)

# カレントディレクトリをスクリプトのディレクトリへ移動
# loggingを初期化(logging.iniファイルがある場合)
os.chdir(os.path.dirname(os.path.abspath(sys.argv[0])))
if os.path.exists("logging.ini"): logging.config.fileConfig("logging.ini")
logger = logging.getLogger(SCRIPT_NAME)

class LocalException(Exception):
    pass

#---------------------------------------
# メイン
#---------------------------------------
def main():
    logger.info(SCRIPT_NAME)

    # 引数解析
    parser = argparse.ArgumentParser(
            formatter_class=argparse.RawTextHelpFormatter,
            description="Dockerコンテナでコマンドを実行します。")

    parser.add_argument(
            "-i", dest="ip_addr", metavar="IP_ADDRESS",
            default=DEFAULT_HOST,
            help="Docker Remote APIの接続先IPアドレス。(default: '%s')" % DEFAULT_HOST)
    parser.add_argument(
            "-p", dest="port", metavar="DOCKER_PORT",
            default=DEFAULT_PORT,
            help="Docker Remote APIの接続先ポート。(default: '%s')" % DEFAULT_PORT)
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
            "-c", dest="cid", metavar="CONTAINER_ID", 
            help="コンテナのIDパターンを指定します。")
    group.add_argument(
            "-n", dest="name", metavar="CONTAINER_NAME",
            help="コンテナの名称パターンを指定します。")

    args = parser.parse_args()

    # Docker接続先URLの構築
    args.url = "http://%s:%s" % (args.ip_addr, args.port)

    # 引数を記録
    logger.info("args:%s", args)

    return start_by_id(args.url, args.cid) if args.cid else start_by_name(args.url, args.name)


def start_by_id(docker_url, id_pattern):
    dc = docker.Client(base_url=docker_url, version=DOCKER_API_VERSION)
    pa = re.compile(('%s.*' % id_pattern[:len(id_pattern) - 1]) if id_pattern.endswith('*') else id_pattern)
    
    def start_container_by_id(c):
      if (not pa.match(c['Id'])):
          return False
      
      try:
          dc.start(c['Id'])
          print '%s %s %s' % (c['Id'][:12], c['Names'][0][1:], 'Success')
          return True
      except Exception, e:
          print '%s %s %s' % (c['Id'][:12], c['Names'][0][1:], 'Fail')
          print e
          return False

    r = map(start_container_by_id, dc.containers(filters={'status':'exited'}))
    code = 0 if r.count(True) == len(r) else (9 if r.count(False) == len(r) else 1)
    #logger.info("exsit code:%s", code)
    return code
  
  
def start_by_name(docker_url, name_pattern):
    dc = docker.Client(base_url=docker_url, version=DOCKER_API_VERSION)
    pa = re.compile(('%s.*' % name_pattern[:len(name_pattern) - 1]) if name_pattern.endswith('*') else name_pattern)
    
    def start_container_by_name(c):
      # 名前に含まれる '/' を除外するため、マッチングする文字列の 1 文字目をスキップ
      matched_names = [n[1:] for n in c['Names'] if pa.match(n[1:])]
      if (len(matched_names) == 0):
          return False
      
      try:
          dc.start(matched_names[0])
          print '%s %s %s' % (c['Id'][:12], matched_names[0], 'Success')
          return True
      except Exception, e:
          print '%s %s %s' % (c['Id'][:12], matched_names[0], 'Fail')
          print e
          return False

    r = map(start_container_by_name, dc.containers(filters={'status':'exited'}))
    code = 0 if r.count(True) == len(r) else (9 if r.count(False) == len(r) else 1)
    #logger.info("exsit code:%s", code)
    return code


#===============================================================================
if __name__ == "__main__":
    try:
        sys.exit(main())

    # 予期されている例外はメッセージのみ出力
    except LocalException as e:
        print e.message
        sys.exit(1)

    # 予期しない例外はスタックトレースまで出力
    except Exception as e:
        print e.message
        traceback.print_exc()
        sys.exit(1)